<?php
/*
  Plugin Name: Pinwheel Hosted Payment Gateway
  Plugin URI:
  Description: Allows you to use Pinwheel Hosted payment page method with the WooCommerce plugin.
  Version: 2.0.0
  Author: By Pinwheel Team
  Author URI: https://www.PinWheelPay.com
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/* Add a custom payment class to WC
  ------------------------------------------------------------ */
add_action('plugins_loaded', 'woocommerce_pinwheelpayapp', 0);

// Change this to according to staging if Environment is other than Production.
$api_local_url = "http://localhost:50001";
$api_staging_url = "https://api.staging.pinwheelportal.com";
$api_prod_url = "https://api.paybypinwheel.com";
$api_ngrok_url = "https://5fdbda5c8cb0.ngrok.io";

//$WP_SITE_URL = "http://192.168.10.25/pinwheel"; //get_site_url();
$WP_SITE_URL = get_site_url();
// $API_URL = $api_ngrok_url;
$API_URL = $api_prod_url;

function woocommerce_pinwheelpayapp(){
    if (!class_exists('WC_Payment_Gateway'))
        return; // if the WC payment gateway class is not available, do nothing
    if(class_exists('WC_pinwheelpayapp'))
        return;

    class WC_Gateway_pinwheelpayapp extends WC_Payment_Gateway{

        // Logging
        public static $log_enabled = false;
        public static $log = false;

        public function __construct(){

            $plugin_dir = plugin_dir_url(__FILE__);

            global $woocommerce, $WP_SITE_URL;

            $this->id = 'pinwheelpayapp';
            $this->icon = apply_filters('woocommerce_pinwheelpayapp_icon', ''.$plugin_dir.'pinwheel-logo.png');
            $this->has_fields = true;

            // Load the settings
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables
            $this->title = $this->get_option('title');
            $this->merchant_id = $this->get_option('merchant_id');
            $this->merchant_slt = $this->get_option('merchant_slt');
            $this->description = $this->get_option('description');
            $this->method_description = "Pinwheel Hosted Gateway redirects customer to Pinwheel to process payment.";
            // $this->notify_url = str_replace( 'https:', 'http:', add_query_arg( 'wc-api', 'WC_Gateway_pinwheelpayapp', home_url( '/' ) ) );
            $this->notify_url = $WP_SITE_URL . "/wp-json/pinwheel/v1/order-updated";
            $this->debug = $this->get_option('debug');

            self::$log_enabled = $this->debug;

            // Actions
            add_action('woocommerce_receipt_' . $this->id, array($this, 'receipt_page'));

            // Save options
            add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

            // Payment listener/API hook
            add_action( 'woocommerce_api_wc_gateway_pinwheelpayapp', array( $this, 'check_ipn_response' ) );

            // Adding Stylesheet
            wp_enqueue_style( 'pinwheel-style', $plugin_dir.'pinwheel-style.css' );

            // if (!$this->is_valid_for_use()){
            //     if ( empty( $log ) ) {
            //         $log = new WC_Logger();
            //     }
            //     $log->add( 'pinwheelpayapp', "Error in Enabling Plugin, Currencies Not Supported");
            //     $this->enabled = false;
            // }
        }

        /**
        * Logging method
        * @param  string $message
        */
        public static function log( $message ) {
            if ( self::$log_enabled ) {
                if ( empty( self::$log ) ) {
                    self::$log = new WC_Logger();
                }
                self::$log->add( 'pinwheelpayapp', $message );
            }
        }

        /**
         * Check if this gateway is enabled and available in the user's country
         *
         * @access public
         * @return bool
         */
        // function is_valid_for_use() {
        //     if ( ! in_array( get_woocommerce_currency(), apply_filters( 'woocommerce_pinwheelpayapp_supported_currencies', array('AUD', 'CAD','USD','EUR','GBP') ) ) ) return false;

        //     return true;
        // }

        /**
         * Admin Panel Options
         * - Options for bits like 'title' and availability on a country-by-country basis
         *
         * @since 1.0.0
         */
        public function admin_options() {

            ?>
            <h3><?php _e( 'Pinwheel Hosted Payment Gateway', 'pinwheelpayapp' ); ?></h3>
            <p><?php _e( 'Pinwheel Settings', 'pinwheelpayapp' ); ?></p>

                <table class="form-table">
                    <?php
                    // Generate the HTML For the settings form.
                    $this->generate_settings_html();
                    ?>
                </table><!--/.form-table-->
                
            <?php
        }


        /**
         * Initialise Gateway Settings Form Fields
         *
         * @access public
         * @return void
         */
        function init_form_fields() {

            $this->form_fields = array(
                'enabled' => array(
                    'title' => __( 'Enable/Disable', 'pinwheelpayapp' ),
                    'type' => 'checkbox',
                    'label' => __( 'Enable Pinwheel Hosted Payment Gateway', 'pinwheelpayapp' ),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __( 'Title', 'pinwheelpayapp' ),
                    'type' => 'text',
                    'description' => __( 'This controls the title which the user sees during checkout.', 'pinwheelpayapp' ),
                    'default' => __( 'Pinwheel', 'pinwheelpayapp' ),
                    'desc_tip'      => true,
                ),
                'description' => array(
                    'title' => __( 'Description', 'pinwheelpayapp' ),
                    'type' => 'textarea',
                    'description' => __( 'This controls the description which the user sees during checkout.', 'pinwheelpayapp' ),
                    'default' => __( 'Pay with Pinwheel', 'pinwheelpayapp' )
                ),
                'merchant_id' => array(
                    'title' => __( 'Merchant Email', 'pinwheelpayapp' ),
                    'type'          => 'text',
                    'description' => __( 'Please enter your Pinwheel Merchant Email.', 'pinwheelpayapp' ),
                    'default' => '',
                    'desc_tip'      => true,
                    'placeholder'   => ''
                ),
                'merchant_slt' => array(
                    'title' => __( 'Merchant Password', 'pinwheelpayapp' ),
                    'type'          => 'text',
                    'description' => __( 'Please enter your Pinwheel Merchant Password.', 'pinwheelpayapp' ),
                    'default' => '',
                    'desc_tip'      => true,
                    'placeholder'   => ''
                ),
                'debug' => array(
                    'title'       => __( 'Debug Log', 'pinwheelpayapp' ),
                    'type'        => 'checkbox',
                    'label'       => __( 'Enable logging', 'pinwheelpayapp' ),
                    'default'     => 'no',
                    'description' => sprintf( __( 'Log Pinwheel events', 'pinwheelpayapp' ), wc_get_log_file_path( 'pinwheelpayapp' ) )
                )
            );

        }

        /**
         * Get Pinwheel Args
         *
         * @access public
         * @param mixed $order
         * @return array
         */
        function get_pwpcheckout_args( $order ) {
            global $woocommerce, $WP_SITE_URL;

            $order_id = $order->id;

            if ( 'yes' == $this->debug )
                $this->log('Generating payment form for order ' . $order->get_order_number());

            $signature = md5($this->merchant_slt);
            $pwpcheckout_args = array();

            $pwpcheckout_args['req_username']       = $this->merchant_id;
            $pwpcheckout_args['signature']          = $signature;
            $pwpcheckout_args['cartName']           = 'Checkout-Direct';
            $pwpcheckout_args['authenticity_token'] = "kcDyh8jCxl6IL3SCDVBtCjTZjMbDQ/7uT9tdit2hsUiPHiFkcC4lMJsUt6BX2OeKYcuJYTR+ENYmyjWpdsEe8g==";
            $pwpcheckout_args['order_id']           = $order_id;
            $pwpcheckout_args['request_amount']     = $order->get_total();
            $pwpcheckout_args['x_url_cancel']       = $order->get_cancel_order_url();
            $pwpcheckout_args['callback_url']       = $this->notify_url;
            $pwpcheckout_args['redirect_success']   = $WP_SITE_URL . '/wp-json/pinwheel/v1/redirect-pending/';
            $pwpcheckout_args['x_url_failed']       = $WP_SITE_URL . '/wp-json/pinwheel/v1/redirect-pending/';
            $pwpcheckout_args['x_url_complete']     = add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay' ))));
            $pwpcheckout_args['request_currency']   = get_woocommerce_currency();
            $pwpcheckout_args['cust_fname']         = $order->billing_first_name;
            $pwpcheckout_args['cust_lname']         = $order->billing_last_name;
            $pwpcheckout_args['bill_addr']          = $order->billing_address_1;
            $pwpcheckout_args['bill_addr_2']        = $order->billing_address_2;
            $pwpcheckout_args['bill_addr_city']     = $order->billing_city;
            $pwpcheckout_args['bill_addr_state']    = $order->billing_state;
            $pwpcheckout_args['bill_addr_country']  = $order->billing_country;
            $pwpcheckout_args['bill_addr_zip']      = $order->billing_postcode;
            $pwpcheckout_args['cust_phone']         = $order->billing_phone;
            $pwpcheckout_args['cust_email']         = $order->billing_email;

            $pwpcheckout_args = apply_filters( 'woocommerce_pinwheelpayapp_args', $pwpcheckout_args );

            return $pwpcheckout_args;
        }

        /**
         * Generate the pwpcheckout button link
         *
         * @access public
         * @param mixed $order_id
         * @return string
         */
        function generate_pwpcheckout_form( $order_id ) {
            global $woocommerce, $API_URL;

            $order = new WC_Order( $order_id );

            $pwpcheckout_args = $this->get_pwpcheckout_args( $order );

            $pwpcheckout_args_array = array();

            foreach ($pwpcheckout_args as $key => $value) {
                $pwpcheckout_args_array[] = '<input type="hidden" name="'.esc_attr( $key ).'" value="'.esc_attr( $value ).'" />';
            }

            wc_enqueue_js( '
            jQuery("body").block({
                    message: "' . esc_js( __( 'Thank you for your order. We are now redirecting you to Pinwheel Payment Page to make payment.', 'pinwheelpayapp' ) ) . '",
                    baseZ: 99999,
                    overlayCSS:
                    {
                        background: "#fff",
                        opacity: 0.6
                    },
                    css: {
                        padding:        "20px",
                        zindex:         "9999999",
                        textAlign:      "center",
                        color:          "#555",
                        border:         "3px solid #aaa",
                        backgroundColor:"#fff",
                        cursor:         "wait",
                        lineHeight:     "24px",
                    }
                });
                jQuery("#submit_pwpcheckout_payment_form").click();
            ' );

            return '<form action="'.$API_URL.'/front-calls/go-in/checkout" method="post" id="pwp_payment_form" target="_top">
                ' . implode( '', $pwpcheckout_args_array) . '
                <input type="submit" class="button alt" id="submit_pwpcheckout_payment_form" value="' . __( 'Pay via Pinwheel', 'pinwheelpayapp' ) . '" /> <a class="button cancel" href="'.esc_url( $order->get_cancel_order_url() ).'">'.__( 'Cancel order &amp; restore cart', 'pinwheelpayapp' ).'</a>
            </form>';

        }

        /**
         * Process the payment and return the result
         *
         * @access public
         * @param int $order_id
         * @return array
         */
        function process_payment( $order_id ) {
            global $woocommerce;

            $order = new WC_Order($order_id);

            if ( 'yes' == $this->debug )
                $this->log( 'Generating Notify payment form for order ' . $order->get_order_number() . '. Notify URL: ' . $this->notify_url . ' REDIRECT URL: ' . add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay' )))) );

            return array(
                'result'    => 'success',
                'redirect'  => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay' ))))
            );

        }

        /**
         * Output for the order received page.
         *
         * @access public
         * @return void
         */
        function receipt_page( $order ) {

            echo '<p>'.__( 'Thank you for your order, please click the button below to pay with Pinwheel.', 'pinwheelpayapp' ).'</p>';

            echo $this->generate_pwpcheckout_form( $order );

        }

        /**
         * Check for Pinwheel Response
         *
         * @access public
         * @return void
         */
        function check_ipn_response() {
            global $woocommerce;
            @ob_clean();

            $wc_order_id    = $_REQUEST['XTL_ORDER_ID'];

            if ( 'yes' == $this->debug )
                $this->log( 'Completing Order: ' . $wc_order_id );

            // $compare_string = $this->secret_word . $this->merchant_id . $_REQUEST['order_number'] . $_REQUEST['total'];
            // $compare_hash1 = strtoupper(md5($compare_string));
            // $compare_hash2 = $_REQUEST['key'];
            // if ($compare_hash1 != $compare_hash2) {
            //     wp_die( "Pinwheel Hash Mismatch... check your secret word." );
            // } else {
                $wc_order   = new WC_Order( absint( $wc_order_id ) );

                // Mark order complete
                $wc_order->payment_complete();

                // Empty cart and clear session
                $woocommerce->cart->empty_cart();

                wp_redirect( $this->get_return_url( $wc_order ) );
                exit;
            // }

        }

        

    }

    /**
     * Add the gateway to WooCommerce
     **/
    function add_pinwheelpayapp_gateway($methods){
        $methods[] = 'WC_Gateway_pinwheelpayapp';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'add_pinwheelpayapp_gateway');

     

}
// API endpoint for redirect
add_action('rest_api_init', function () {
    register_rest_route( 'pinwheel/v1', 'redirect-pending/',array(
                'methods'  => 'GET',
                'callback' => 'get_redirect_order'
    ));
});

function get_redirect_order($request) {
    global $woocommerce, $WP_SITE_URL;
    try{
        // $args = array(
        //     'req_username' => $request['req_username'],
        //     'req_password' => $request['req_password']
        // );

        // $payment_gateway = WC()->payment_gateways->payment_gateways()['pinwheelpayapp'];

        // $req_username = $payment_gateway->req_username;
        // $req_password = $payment_gateway->req_password;

        // if(strcmp($req_username, $args['req_username']) !== 0){
        //     return new WP_Error( 'pinwheel_merchant_error', 'incorrect credentials', array('status' => 404) );
        // }

        // if(strcmp($req_password, $args['req_password']) !== 0){
        //     return new WP_Error( 'pinwheel_merchant_error', 'incorrect credentials', array('status' => 404) );
        // }

        $order_id = $request['XTL_ORDER_ID'];
        $order_complete_url = $request['TRANS_COMPLETE_URL'];

        // if ( 'yes' == $this->debug )
        //     $this->log( 'Completing Order: ' . $wc_order_id );
        
        $order = new WC_Order( $order_id );
        $status = strtoupper($request['TRANS_STATUS_NAME']);

        if ( empty( $log ) ) {
            $log = new WC_Logger();
        }
        $log->add( 'pinwheelpayapp', "Redirecting for Order Status => " . $status . "---" . $order_id );
        

        if($status == 'APPROVED'){
            $order->add_order_note( 'Pinwheel Transaction Completed and Transaction Id: ' . $request['TRANS_ORDER_ID'] );
            add_post_meta( $order->id, '_transaction_id', $request['TRANS_ORDER_ID'], true );
            $order->payment_complete( $request['TRANS_ORDER_ID'] );
            $order->reduce_order_stock();

        }else if($status == 'PENDING'){
            $order->add_order_note( 'Pinwheel Transaction is Pending now and Transaction Id: ' . $request['TRANS_ORDER_ID'] .'. We Strongly recommend do not ship order yet.');
            add_post_meta( $order->id, '_transaction_id', $request['TRANS_ORDER_ID'], true );
            $order->update_status('on-hold');
        }else if($status == 'DECLINED'){
            $order->add_order_note( 'Pinwheel Transaction is FAILED and Transaction Id: ' . $request['TRANS_ORDER_ID'] .'. We Strongly recommend do not ship order.');
            $order->add_order_note( 'Pinwheel Transaction is DECLINED due to: ' . $request['ERROR_MESSAGE'] .', API-ADVICE: ' . $request['API_ADVICE'] . '. We Strongly recommend do not ship order.');
            add_post_meta( $order->id, '_transaction_id', $request['TRANS_ORDER_ID'], true );
            $order->update_status('failed');
            wp_redirect( $order->get_cancel_order_url() );
        }
    }
    catch ( Exception $ex ) { // Add log
        if ( empty( $log ) ) {
            $log = new WC_Logger();
        }
        $log->add( 'pinwheelpayapp', $ex->getMessage() . ", Order ID: " . $order_id );
    }

    // Mark order complete
    // $wc_order->payment_complete();

    // Empty cart and clear session
    // 

    wp_redirect( $order_complete_url );
    exit;

}

// API endpoint for STATUS
add_action('rest_api_init', function () {
    register_rest_route( 'pinwheel/v1', 'order-updated/',array(
                'methods'  => 'POST',
                'callback' => 'order_status_changed'
    ));
});

function order_status_changed($request) {
    // $args = array(
    //     'req_username' => $request['req_username'],
    //     'req_password' => $request['req_password']
    // );

    // $payment_gateway = WC()->payment_gateways->payment_gateways()['pinwheelpayapp'];

    // $req_username = $payment_gateway->req_username;
    // $req_password = $payment_gateway->req_password;

    // if(strcmp($req_username, $args['req_username']) !== 0){
    //     return new WP_Error( 'pinwheel_merchant_error', 'incorrect credentials', array('status' => 404) );
    // }

    // if(strcmp($req_password, $args['req_password']) !== 0){
    //     return new WP_Error( 'pinwheel_merchant_error', 'incorrect credentials', array('status' => 404) );
    // }

    global $woocommerce;

    $order_id = $request['XTL_ORDER_ID'];

    
    $order = new WC_Order( $order_id );
    $status = strtoupper($request['TRANS_STATUS_NAME']);
    
    if ( empty( $log ) ) {
        $log = new WC_Logger();
    }
    $log->add( 'pinwheelpayapp', "Changing Order Status to => " . $status . "--- For Order ID: " . $order_id );

    if($status == 'APPROVED'){
        $order->add_order_note( 'Pinwheel Transaction Completed and Transaction Id: ' . $request['TRANS_ORDER_ID'] );
        add_post_meta( $order->id, '_transaction_id', $request['TRANS_ORDER_ID'], true );
        $order->payment_complete( $request['TRANS_ORDER_ID'] );
        $order->reduce_order_stock();
    }else if($status == 'PENDING'){
        $order->add_order_note( 'Pinwheel PENDING Transaction and Transaction Id: ' . $request['TRANS_ORDER_ID'] );
        add_post_meta( $order->id, '_transaction_id', $request['TRANS_ORDER_ID'], true );
        $order->update_status('on-hold');
    }else if($status == 'DECLINED'){
        $order->add_order_note( 'Pinwheel FAILED Transaction and Transaction Id: ' . $request['TRANS_ORDER_ID'] );
        $order->add_order_note( 'Pinwheel Transaction is DECLINED due to: ' . $request['ERROR_MESSAGE'] .', API-ADVICE: ' . $request['API_ADVICE'] . '. We Strongly recommend do not ship order.');
        add_post_meta( $order->id, '_transaction_id', $request['TRANS_ORDER_ID'], true );
        $order->update_status('failed');
    }

    return $request;
    exit;

}
